unit uMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Math,
  Forms, Controls, ComCtrls, StdCtrls, ExtCtrls;

type

  { TMainForm }

  TMainForm = class(TForm)
    btnSortData: TButton;
    btnNewData: TButton;
    btnSortListView: TButton;
    cbVirtualMode: TCheckBox;
    ListView: TListView;
    Panel1: TPanel;
    procedure btnNewDataClick(Sender: TObject);
    procedure btnSortDataClick(Sender: TObject);
    procedure btnSortListViewClick(Sender: TObject);
    procedure cbVirtualModeChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ListViewCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure ListViewData(Sender: TObject; Item: TListItem);
  private
    FSeed: Integer;
    procedure Prepare(VirtualMode: Boolean);
  end;

var
  MainForm: TMainForm;
  Data: TStringList;

implementation

{$R *.lfm}

const
  COUNT = 10;

function RandomString(Base: String; DateRange: Integer = 0): String;
begin
  if DateRange <= 0 then DateRange := 1000;
  Result := Base + '_' + DateToStr(Date() - Random(DateRange));
end;

function ExtractDate(s: String): TDate;
var
  i: Integer;
  tmp: String;
begin
  Result := 0;
  tmp := '';
  for i := Length(s) downto 1 do
    if (s[i] in ['0'..'9', '.']) then
      tmp := s[i] + tmp
    else
      break;
  Result := StrToDate(tmp);
end;

function StringListSortCompare(List: TStringList; Index1, Index2: Integer): Integer;
begin
  Result := CompareValue(ExtractDate(List[Index1]), ExtractDate(List[Index2]));
end;


{  TMainForm }

procedure TMainForm.btnNewDataClick(Sender: TObject);
begin
  FSeed := Random(1000);
  Prepare(cbVirtualMode.Checked);
end;

procedure TMainForm.btnSortDataClick(Sender: TObject);
begin
  if Assigned(Data) then
  begin
    Data.CustomSort(@StringListSortCompare);
    ListView.Invalidate;
  end;
end;

procedure TMainForm.btnSortListViewClick(Sender: TObject);
begin
  ListView.AlphaSort;
end;

procedure TMainForm.cbVirtualModeChange(Sender: TObject);
begin
  Prepare(cbVirtualMode.Checked);
  btnSortData.Enabled := cbVirtualMode.Checked;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Prepare(cbVirtualMode.Checked);
  ListView.SortType := stText;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Data.Free;
end;

procedure TMainForm.ListViewCompare(Sender: TObject; Item1, Item2: TListItem;
  Data: Integer; var Compare: Integer);
begin
  Compare := CompareValue(ExtractDate(Item1.Caption), ExtractDate(Item2.Caption));
end;

procedure TMainForm.ListViewData(Sender: TObject; Item: TListItem);
begin
  Item.Caption := Data[Item.Index];
end;

procedure TMainForm.Prepare(VirtualMode: Boolean);

  function MakeString: String;
  const
    ItemBase: array[0..4] of string = ('Brief', 'Rechnung', 'Mahnung', 'Protokoll', 'Notiz');
  begin
    Result := RandomString(ItemBase[Random(Length(ItemBase))], 5000);
  end;

var
  i: Integer;
begin
  FreeAndNil(Data);
  ListView.Items.Clear;

  RandSeed := FSeed;
  if VirtualMode then
  begin
    Data := TStringList.Create;
    for i := 0 to COUNT-1 do
      Data.Add(MakeString);

    ListView.OwnerData := true;
    ListView.OnData := @ListViewData;
    ListView.Items.Count := Data.Count;
  end else
  begin
    ListView.OwnerData := false;
    ListView.OnData := nil;
    for i := 0 to COUNT-1 do
      ListView.Items.Add.Caption := MakeString;
  end;
end;

end.

